/*
 *   Copyright (C) by Y.Ishioka.
 */
#include "STM32F7xx/Include/stm32f7xx.h"
#include "stm32f7xx_hal.h"
#include "stm32746g_discovery.h"
#include <string.h>

#include "matled_param.h"


void  syslog(unsigned long prio, const char *format, ...);
int  dly_tsk( unsigned long );
void  busyloop_usec( int );

void  lcd_draw_erase_draw( void );
void  lcd_draw_slide_draw( void );
void  lcd_pset( int x, int y, int color );


struct lcd_draw_col_pos_tag  lcd_draw_col_pos[8] = {
    { DEF_COL_BASE_X1,     DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+50,  DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+100, DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+150, DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+200, DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+250, DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+300, DEF_COL_BASE_Y1 },
    { DEF_COL_BASE_X1+350, DEF_COL_BASE_Y1 }
};


unsigned short int  color_rgb565[] = {
    DEF_RGB565_RED,
    DEF_RGB565_GRN,
    DEF_RGB565_BLU,
    DEF_RGB565_RED+DEF_RGB565_GRN,
    DEF_RGB565_RED+DEF_RGB565_BLU,
    DEF_RGB565_GRN+DEF_RGB565_BLU,
    0xffff,
    0x0000
};

unsigned char  matled_frame[ DEF_DISP_HIGHT ][ DEF_FLMBUF_LEN ];
int  matled_disp_posi;
int  matled_erase_mode;
int  matled_slide_mode;

int  lcd_dot_color;


extern  const unsigned char  fx_8x16rk_fnt[];
extern  const unsigned char  jiskan16_fnt[];
unsigned char  dat_bit[8] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 } ;

unsigned char  *get_fontx2_a_x( unsigned int code )
{
    unsigned char  *font = (unsigned char *)fx_8x16rk_fnt;
    unsigned char  *address;
    unsigned int  fontbyte;

    fontbyte = (font[14] + 7) / 8 * font[15];
    address = &font[17] + fontbyte * code;

    return  address;
}


void  set_font_a( int x, int y, int color, unsigned char code )
{
    int  i, j ;
    unsigned char  *font;

    font = get_fontx2_a_x( code );

    for( i=0 ; i<16 ; i++ ) {  /* line */
        for( j=0 ; j<8 ; j++ ) {
            if( (font[i] & dat_bit[j]) != 0 ) {
                lcd_pset( x+j, y+i, color );
            }
        }
    }
}


unsigned char  *get_fontx2_k_x( unsigned int code )
{
    unsigned char  *font = (unsigned char *)jiskan16_fnt;
    unsigned char  *address = NULL ;
    unsigned char  *tmp ;
    unsigned int  blknum, i, fontnum ;
    unsigned int  bstart, bend ;
    unsigned int  fontbyte ;

    fontbyte = (font[14] + 7) / 8 * font[15] ;
    fontnum = 0 ;

    blknum = (unsigned int)font[17] * 4 ;
    tmp = &font[18] ;
    for( i=0 ; i<blknum ; i+=4 ) {
        bstart = tmp[i]   + ((unsigned int)tmp[i+1] << 8) ;
        bend   = tmp[i+2] + ((unsigned int)tmp[i+3] << 8) ;
        if( code >= bstart && code <= bend ) {
            address = tmp + (fontnum + (code - bstart)) * fontbyte + blknum ;
            break ;
        }

        fontnum += (bend - bstart) + 1 ;
    }

    return  address ;
}


void  set_font_k( int x, int y, int color, unsigned int code )
{
    int  i, j, k;
    unsigned char  *font;

    font = get_fontx2_k_x( code );

    for( i=0 ; i<16 ; i++ ) {  /* line */
        for( k=0 ; k<2 ; k++ ) {
            for( j=0 ; j<8 ; j++ ) {
                if( (font[i*2 + k] & dat_bit[j]) != 0 ) {
                    lcd_pset( x+8*k+j, y+i, color );
                }
            }
        }
    }

}


void  set_string( int x, int y, int color, char *buff, int size )
{
    int  i;
    unsigned int code;
    unsigned char *pnt;

    for( i=0 ; i<size ; i++ ) {
        if( *buff < 0x80 ) {
            set_font_a( x, y, color, (unsigned char)*buff );
            x += 8;
        } else {
            pnt = (unsigned char *)buff;
            code = *pnt<<8;
            pnt++;
            code += *pnt;
            set_font_k( x, y, color, code );
            x += 16;
            i++;
            buff++;
        }
        buff++;
    }
}


void  set_string_demo( int x, int y, int color, char *buff, int size )
{
    int  i;
    unsigned int code;
    unsigned char *pnt;

    for( i=0 ; i<size ; i++ ) {
        if( color > 6 ) {
            color = 0;
        }
        if( *buff < 0x80 ) {
            set_font_a( x, y, color, (unsigned char)*buff );
            x += 8;
        } else {
            pnt = (unsigned char *)buff;
            code = *pnt<<8;
            pnt++;
            code += *pnt;
            set_font_k( x, y, color, code );
            x += 16;
            i++;
            buff++;
        }
        buff++;
        color++;
    }
}


void  matled_dev_out( unsigned char *buff, int posx )
{
    int  x, y ;
    unsigned char  data ;
    unsigned char  *pnt ;

    for( y=0 ; y<16 ; y++ ) {

        for( x=0 ; x<DEF_DISP_WIDTH ; x++ ) {

            GPIOF->BSRR = 0x0700 << 16 ; /* off R1 G1 B1 */
            GPIOI->BSRR = 0x000d << 16 ; /* off R2 G2 B2 */
            GPIOH->BSRR = 0x0040 << 16 ; /* off R2X */

            /* LINE1 */
            /** set R1 G1 B1 **/
            pnt = (unsigned char *)matled_frame ;
            pnt += (y * DEF_FLMBUF_LEN) + x + matled_disp_posi;
            //syslog(3, "pnt1=0x%08x", (unsigned int)pnt );
            data = *pnt;
            if( data & 0x01 ) {
                GPIOF->BSRR = 0x0100;
            }
            if( data & 0x02 ) {
                GPIOF->BSRR = 0x0200;
            }
            if( data & 0x04 ) {
                GPIOF->BSRR = 0x0400;
            }

            /* LINE2 */
            /** set R2 G2 B2 **/
            pnt = (unsigned char *)matled_frame ;
            pnt += ((y+16) * DEF_FLMBUF_LEN) + x + matled_disp_posi;
            //syslog(3, "pnt2=0x%08x", (unsigned int)pnt );
            data = *pnt;
            if( data & 0x01 ) {
                GPIOI->BSRR = 0x0001;
                GPIOH->BSRR = 0x0040; /* on R2X */
            }
            if( data & 0x02 ) {
                GPIOI->BSRR = 0x0004;
            }
            if( data & 0x04 ) {
                GPIOI->BSRR = 0x0008;
            }

            /* toggle CLK */
            GPIOB->BSRR = 0x0010 ;       /* on  CLK */
            busyloop_usec(1);
            GPIOB->BSRR = 0x0010 << 16 ; /* off CLK */
            busyloop_usec(1);
        }

        GPIOA->BSRR = 0x8100 ;       /* on  LAT, on  OE */
        GPIOF->BSRR = 0x0080 ;       /* on  OEX */

        /* set A,B,C,D */
        GPIOB->BSRR = 0xc000 << 16 ;     /* off A B */
        GPIOG->BSRR = 0x00c0 << 16 ;     /* off C D */
        GPIOB->BSRR = 0xc000 & (y<<14) ; /* set A B */
        GPIOG->BSRR = 0x00c0 & (y<<4);   /* set C D */
        busyloop_usec(1);

        GPIOA->BSRR = 0x8100 << 16 ; /* off LAT, off OE */
        GPIOF->BSRR = 0x0080 << 16 ; /* off OEX */
        busyloop_usec(1);
    }

    for( x=0 ; x<DEF_DISP_WIDTH ; x++ ) {
        GPIOF->BSRR = 0x0700 << 16 ; /* off R1 G1 B1 */
        GPIOI->BSRR = 0x000d << 16 ; /* off R2 G2 B2 */
        GPIOH->BSRR = 0x0040 << 16 ; /* off R2X */
        /* toggle CLK */
        GPIOB->BSRR = 0x0010 ;       /* on  CLK */
        busyloop_usec(1);
        GPIOB->BSRR = 0x0010 << 16 ; /* off CLK */
        busyloop_usec(1);
    }

    GPIOA->BSRR = 0x8100 ;       /* on  LAT, on  OE */
    GPIOF->BSRR = 0x0080 ;       /* on  OEX */
    busyloop_usec(1);
    GPIOA->BSRR = 0x8100 << 16 ; /* off LAT, off OE */
    GPIOF->BSRR = 0x0080 << 16 ; /* off OEX */

    dly_tsk(1);
}


void  matled_dev_init( void )
{
    int  i, j ;

    /* set clock enable */
    RCC->AHB1ENR |= ( RCC_AHB1ENR_GPIOAEN
                    | RCC_AHB1ENR_GPIOBEN
                    | RCC_AHB1ENR_GPIOFEN
                    | RCC_AHB1ENR_GPIOGEN
                    | RCC_AHB1ENR_GPIOHEN
                    | RCC_AHB1ENR_GPIOIEN
                    ) ;

    /** for R1:PF8 G1:PF9 B1:PF10 **/
    GPIOF->MODER   &= ~(0x003f0000);
    GPIOF->MODER   |=  (0x00150000);
    GPIOF->OTYPER  &= ~(0x0700);
    GPIOF->OSPEEDR &= ~(0x003f0000);
    GPIOF->OSPEEDR |=  (0x002a0000);
    GPIOF->PUPDR   &= ~(0x003f0000);
    GPIOF->BSRR = 0x0700 << 16 ; /* off R1 G1 B1 */

    /** for R2:PI0 G2:PI2 B2:PI3 **/
    GPIOI->MODER   &= ~(0x000000f3);
    GPIOI->MODER   |=  (0x00000051);
    GPIOI->OTYPER  &= ~(0x000d);
    GPIOI->OSPEEDR &= ~(0x000000f3);
    GPIOI->OSPEEDR |=  (0x000000a2);
    GPIOI->PUPDR   &= ~(0x000000f3);
    GPIOI->BSRR = 0x000d << 16 ; /* off R2 G2 B2 */

    /** for R2X:PH6 **/
    GPIOH->MODER   &= ~(0x00003000);
    GPIOH->MODER   |=  (0x00001000);
    GPIOH->OTYPER  &= ~(0x0040);
    GPIOH->OSPEEDR &= ~(0x00003000);
    GPIOH->OSPEEDR |=  (0x00002000);
    GPIOH->PUPDR   &= ~(0x00003000);
    GPIOH->BSRR = 0x0040 << 16 ; /* off R2X */

    /** for A:PB14 B:PB15 **/
    GPIOB->MODER   &= ~(0xf0000000);
    GPIOB->MODER   |=  (0x50000000);
    GPIOB->OTYPER  &= ~(0xc000);
    GPIOB->OSPEEDR &= ~(0xf0000000);
    GPIOB->OSPEEDR |=  (0xa0000000);
    GPIOB->PUPDR   &= ~(0xf0000000);

    /** for C:PG6 D:PG7 **/
    GPIOG->MODER   &= ~(0x0000f000);
    GPIOG->MODER   |=  (0x00005000);
    GPIOG->OTYPER  &= ~(0x00c0);
    GPIOG->OSPEEDR &= ~(0x0000f000);
    GPIOG->OSPEEDR |=  (0x0000a000);
    GPIOG->PUPDR   &= ~(0x0000f000);

    GPIOB->BSRR = 0xc000 << 16 ; /* off A B */
    GPIOG->BSRR = 0x00c0 << 16 ; /* off C D */

    /** for LAT:PA15 OE:PA8 **/
    GPIOA->MODER   &= ~(0xc0030000);
    GPIOA->MODER   |=  (0x40010000);
    GPIOA->OTYPER  &= ~(0x8100);
    GPIOA->OSPEEDR &= ~(0xc0030000);
    GPIOA->OSPEEDR |=  (0x80020000);
    GPIOA->PUPDR   &= ~(0xc0030000);
    GPIOA->BSRR = 0x8100 << 16 ; /* off LAT OE */

    /** for OEX:PF7 **/
    GPIOF->MODER   &= ~(0x0000c000);
    GPIOF->MODER   |=  (0x00004000);
    GPIOF->OTYPER  &= ~(0x0080);
    GPIOF->OSPEEDR &= ~(0x0000c000);
    GPIOF->OSPEEDR |=  (0x00008000);
    GPIOF->PUPDR   &= ~(0x0000c000);
    GPIOF->BSRR = 0x0080 << 16 ; /* off OEX */

    /** for CLK PB4 **/
    GPIOB->MODER   &= ~(0x00000300);
    GPIOB->MODER   |=  (0x00000100);
    GPIOB->OTYPER  &= ~(0x0010);
    GPIOB->OSPEEDR &= ~(0x00000300);
    GPIOB->OSPEEDR |=  (0x00000200);
    GPIOB->PUPDR   &= ~(0x00000300);
    GPIOB->BSRR = 0x0010 << 16;  /* off CLK */

    /* clear matled */
    for( i=0 ; i<16 ; i++ ) {
        for( j=0 ; j<DEF_DISP_WIDTH ; j++ ) {

            GPIOF->BSRR = 0x0700 << 16 ; /* off R1 G1 B1 */
            GPIOI->BSRR = 0x000d << 16 ; /* off R2 G2 B2 */
            GPIOH->BSRR = 0x0040 << 16 ; /* off R2X */

            /* toggle CLK */
            GPIOB->BSRR = 0x0010 ;       /* on  CLK */
            busyloop_usec(1);
            GPIOB->BSRR = 0x0010 << 16 ; /* off CLK */
            busyloop_usec(1);
        }

        GPIOA->BSRR = 0x8100 ;       /* on  LAT, on  OE */
        GPIOF->BSRR = 0x0080 ;       /* on  OEX */

        /* set A,B,C,D */
        GPIOB->BSRR = 0xc000 << 16 ;     /* off A B */
        GPIOG->BSRR = 0x00c0 << 16 ;     /* off C D */
        GPIOB->BSRR = 0xc000 & (i<<14) ; /* set A B */
        GPIOG->BSRR = 0x00c0 & (i<<4);   /* set C D */
        busyloop_usec(1);

        GPIOA->BSRR = 0x8100 << 16 ; /* off LAT, off OE */
        GPIOF->BSRR = 0x0080 << 16 ; /* off OEX */
        busyloop_usec(1);
    }

    /* clear frame buffer */
    for( i=0 ; i<DEF_DISP_HIGHT ; i++  ) {
        for( j=0 ; j<DEF_DISP_WIDTH ; j++ ) {
            matled_frame[ i ][ j ] = 0;
        }
    }
}


void  matled_copy_lcddraw( void )
{
    unsigned int  baseA = 0xc0000000;
    unsigned int  baseB = 0xc0000000;
    unsigned int  baseD = 0xc0000000;
    unsigned short int  *src;
    unsigned short int  *dst;
    int  x, y;

    baseD += (DEF_FRAME1_X1 + 480 * DEF_FRAME1_Y1) * sizeof(unsigned short int);
    baseA += (DEF_FRAMEA_X1 + 480 * DEF_FRAMEA_Y1) * sizeof(unsigned short int);
    baseB += (DEF_FRAMEB_X1 + 480 * DEF_FRAMEB_Y1) * sizeof(unsigned short int);
    for( y=0 ; y<DEF_DISP_HIGHT ; y++ ) {
        dst = (unsigned short int *)(baseD + (480 * y) * sizeof(unsigned short int));
        src = (unsigned short int *)(baseA + (480 * y * 2) * sizeof(unsigned short int));
        for( x=0 ; x<DEF_MATFLM_LEN/2 ; x++ ) {
            *dst++ = *src;
            src += 2;
        }

        src = (unsigned short int *)(baseB + (480 * y * 2) * sizeof(unsigned short int));
        for( x=DEF_MATFLM_LEN/2 ; x<DEF_MATFLM_LEN ; x++ ) {
            *dst++ = *src;
            src += 2;
        }
    }
}


void  matled_copy_lcdframe( void )
{
    unsigned int  base = 0xc0000000;
    unsigned short int  *src;
    unsigned short int  data;
    unsigned char  *dst;
    unsigned char  color;
    int  x, y;

    base += (DEF_FRAME1_X1 + 480 * DEF_FRAME1_Y1) * sizeof(unsigned short int);

    for( y=0 ; y<DEF_DISP_HIGHT ; y++ ) {
        src = (unsigned short int *)(base + (480 * y) * sizeof(unsigned short int));
        dst = &matled_frame[y][DEF_DISP_WIDTH];
        for( x=0 ; x<DEF_MATFLM_LEN ; x++ ) {
            data = *src++;
            color = 0;
            if( data & DEF_RGB565_RED ) { /* check R */
                color |= 0x1;
            }
            if( data & DEF_RGB565_GRN ) { /* check G */
                color |= 0x2;
            }
            if( data & DEF_RGB565_BLU ) { /* check B */
                color |= 0x4;
            }
            *dst++ = color;
        }
    }
}


void  matled_disp_slide( void )
{
    if( matled_slide_mode ) {
        matled_disp_posi++ ;
        if( matled_disp_posi > DEF_FLMBUF_LEN-DEF_DISP_WIDTH) {
            matled_disp_posi = 0;
        }
    }
}


void  matled_set_erase( void )
{
    if( matled_erase_mode ) {
        matled_erase_mode = 0;
    } else {
        matled_erase_mode++;
    }
    syslog(3, "slide mode=%d  posi=%d", matled_slide_mode, matled_disp_posi );
    lcd_draw_erase_draw();
}


void  matled_set_slide( void )
{
    if( matled_slide_mode ) {
        matled_slide_mode = 0;
    } else {
        matled_slide_mode++;
    }
    syslog(3, "slide mode=%d  posi=%d", matled_slide_mode, matled_disp_posi );
    lcd_draw_slide_draw();
}


void  lcd_pset( int x, int y, int color )
{
    unsigned short int  *dst = (void*)0xc0000000;

    dst += x + y * 480;
    *dst = color_rgb565[color];
}


void  lcd_prect( int x, int y, int color )
{
    unsigned short int  *dst = (void*)0xc0000000;
    int  x0, y0;

    x0 = x & ~0x1;
    y0 = y & ~0x1;

    dst += x0 + y0 * 480;
    *dst = color_rgb565[color];
    *(dst+1) = color_rgb565[color];
    dst += 480;
    *dst = color_rgb565[color];
    *(dst+1) = color_rgb565[color];
}


void  lcd_erase( int x0, int y0 )
{
    int  x, y;
    int  xmin, xmax;
    int  ymin, ymax;

    if( x0 < 2 ) {
        xmin = 0;
    } else {
        xmin = x0-2;
    }
    if( x0 >= 480-2 ) {
        xmax = 480-1;
    } else {
        xmax = x0+2;
    }

    if( y0 < DEF_LMT_BASE_Y1+2 ) {
        ymin = DEF_LMT_BASE_Y1;
    } else {
        ymin = y0-2;
    }
    if( y0 >= 272-2 ) {
        ymax = 272-1;
    } else {
        ymax = y0+2;
    }

    for( x=xmin ; x<=xmax ; x+=2 ) {
        for( y=ymin ; y<=ymax ; y+=2 ) {
            lcd_prect( x, y, 7 ); /* set BLACK */
        }
    }
}


void  lcd_x_line( int x0, int x1, int y0, int color )
{
    int x;

    if( x0 > x1 ) {
        x  = x0;
        x0 = x1;
        x1 = x;
    }
    for( x=x0 ; x<=x1 ; x++ ) {
        lcd_prect( x, y0, color );
    }
}


void  lcd_y_line( int x0, int y0, int y1, int color )
{
    int y;

    if( y0 > y1 ) {
        y  = y0;
        y0 = y1;
        y1 = y;
    }
    for( y=y0 ; y<=y1 ; y++ ) {
        lcd_prect( x0, y, color );
    }
}


void  lcd_line( int x0, int y0, int x1, int y1, int color )
{
    int  dx, dy;
    int  steep;
    int  temp;
    int  err;
    int  ystep;

    dx = x1 - x0;
    if( dx == 0 ) {
        lcd_y_line( x0, y0, y1, color );
        return;
    }
    if( dx < 0 ) {
        dx = -dx;
    }

    dy = y1 - y0;
    if( dy == 0 ) {
        lcd_x_line( x0, x1, y1, color );
        return;
    }
    if( dy < 0 ) {
        dy = -dy;
    }
    steep = dy > dx;
    if( steep ) {
        temp = x0;
        x0 = y0;
        y0 = temp;
        temp = x1;
        x1 = y1;
        y1 = temp;
    }

    if( x0 > x1 ){
        temp = x0;
        x0 = x1;
        x1 = temp;
        temp = y0;
        y0 = y1;
        y1 = temp;
    }

    dx = x1 - x0;
    dy = y1 - y0;
    if( dy < 0 ) {
        dy = -dy;
    }

    err = dx / 2;
    if( y0 < y1 ) {
        ystep = 1;
    } else {
        ystep = -1;
    }

    for ( ; x0<=x1 ; x0++ ){
        if( steep ){
            lcd_prect( y0, x0, color );
        } else {
            lcd_prect( x0, y0, color );
        }
        err -= dy;
        if( err < 0 ){
            y0 += ystep;
            err += dx;
        }
    }
}


int  lcd_draw_cont_flag;
int  lcd_draw_bef_x;
int  lcd_draw_bef_y;

void  lcd_draw( int x, int y, int color )
{
    if( matled_erase_mode ) {
        lcd_erase( x, y );
        return;
    }

    if( lcd_draw_cont_flag == 0 ) {
        lcd_prect( x, y, color );
        lcd_draw_bef_x = x;
        lcd_draw_bef_y = y;
        lcd_draw_cont_flag = 1;
    } else {
        lcd_line( lcd_draw_bef_x, lcd_draw_bef_y, x, y, color );
        lcd_draw_bef_x = x;
        lcd_draw_bef_y = y;
    }
}

void  lcd_draw_cont_clear( void )
{
     lcd_draw_cont_flag = 0;
}


void  lcd_draw_drawareaA( void )
{
    int x, y;

    for( x=DEF_FRAMEA_X1-1 ; x<DEF_FRAMEA_X1+DEF_MATFLM_LEN+1 ; x++ ) {
        lcd_pset( x, DEF_FRAMEA_Y1-1, 6 );
        lcd_pset( x, DEF_FRAMEA_Y1+DEF_DISP_HIGHT*2-1+1, 6 );
    }
    for( y=DEF_FRAMEA_Y1-1 ; y<DEF_FRAMEA_Y1+DEF_DISP_HIGHT*2+1 ; y++ ) {
        lcd_pset( DEF_FRAMEA_X1-1,   y, 6 );
        lcd_pset( DEF_FRAMEA_X1+DEF_MATFLM_LEN, y, 6 );
    }
}


void  lcd_clear_drawareaA( void )
{
    int x, y;

    for( x=DEF_FRAMEA_X1 ; x<DEF_FRAMEA_X1+DEF_MATFLM_LEN ; x++ ) {
        for( y=DEF_FRAMEA_Y1 ; y<DEF_FRAMEA_Y1+DEF_DISP_HIGHT*2 ; y++ ) {
            lcd_pset( x, y, 7 );
        }
    }
}


void  lcd_draw_drawareaB( void )
{
    int x, y;

    for( x=DEF_FRAMEB_X1-1 ; x<DEF_FRAMEB_X1+DEF_MATFLM_LEN+1 ; x++ ) {
        lcd_pset( x, DEF_FRAMEB_Y1-1, 6 );
        lcd_pset( x, DEF_FRAMEB_Y1+DEF_DISP_HIGHT*2-1+1, 6 );
    }
    for( y=DEF_FRAMEB_Y1-1 ; y<DEF_FRAMEB_Y1+DEF_DISP_HIGHT*2+1 ; y++ ) {
        lcd_pset( DEF_FRAMEB_X1-1,   y, 6 );
        lcd_pset( DEF_FRAMEB_X1+DEF_MATFLM_LEN, y, 6 );
    }
}


void  lcd_clear_drawareaB( void )
{
    int x, y;

    for( x=DEF_FRAMEB_X1 ; x<DEF_FRAMEB_X1+DEF_MATFLM_LEN ; x++ ) {
        for( y=DEF_FRAMEB_Y1 ; y<DEF_FRAMEB_Y1+DEF_DISP_HIGHT*2 ; y++ ) {
            lcd_pset( x, y, 7 );
        }
    }
}


void  lcd_draw_framearea( void )
{
    int x, y;

    for( x=DEF_FRAME1_X1-1 ; x<DEF_FRAME1_X1+DEF_MATFLM_LEN+1 ; x++ ) {
        lcd_pset( x, DEF_FRAME1_Y1-1, 6 );
        lcd_pset( x, DEF_FRAME1_Y1+DEF_DISP_HIGHT-1+1, 6 );
    }
    for( y=DEF_FRAME1_Y1-1 ; y<DEF_FRAME1_Y1+DEF_DISP_HIGHT+1 ; y++ ) {
        lcd_pset( DEF_FRAME1_X1-1,   y, 6 );
        lcd_pset( DEF_FRAME1_X1+DEF_MATFLM_LEN, y, 6 );
    }

    lcd_draw_drawareaA();
    lcd_draw_drawareaB();
}


void  lcd_clear_framearea( void )
{
    int x, y;

    for( x=DEF_FRAME1_X1 ; x<DEF_FRAME1_X1+DEF_MATFLM_LEN ; x++ ) {
        for( y=DEF_FRAME1_Y1 ; y<DEF_FRAME1_Y1+DEF_DISP_HIGHT ; y++ ) {
            lcd_pset( x, y, 7 );
        }
    }
}


void  lcd_clear_drawall( void )
{
    int x, y;

    for( x=0 ; x<480 ; x++ ) {
        for( y=DEF_LMT_BASE_Y1 ; y<272 ; y++ ) {
            lcd_pset( x, y, 7 );
        }
    }
}


const unsigned char	lcd_init_msg2[] = {
    0x83,0x66,0x83,0x57,0x83,0x5e,0x83,
    0x8b,0x82,0xa8,0x8a,0x47,0x82,0xa9,0x82,0xab,0x83,
    0x7b,0x81,0x5b,0x83,0x68,0x82,0xc5,0x82,0xb7,0x81,
    0x42,0x20,0x8a,0x47,0x82,0xf0,0x95,0x60,0x82,0xa2,
    0x82,0xc4,0x95,0x5c,0x8e,0xa6,0x82,0xb3,0x82,0xb9,
    0x82,0xdc,0x82,0xb5,0x82,0xe5,0x82,0xa4,0x81,0x42,
    0x00
};


void  lcd_init_framearea( void )
{
    unsigned char  *msg;
    int  size;

    lcd_clear_framearea();

    msg = (unsigned char *)lcd_init_msg2;
    size = sizeof( lcd_init_msg2 ) -1;
    set_string_demo( DEF_FRAME1_X1+5, DEF_FRAME1_Y1+8, 0, (char *)msg, size );

    matled_copy_lcdframe();
    matled_disp_posi = 0;
    matled_slide_mode = 0;
    matled_set_slide();
}


void  lcd_draw_use_color_area( void )
{
    int x, y;
    int x1 = DEF_COL_USE_X1;
    int y1 = DEF_COL_USE_Y1;
    int color = 6;
    char *msg = "use";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );
}


void  lcd_draw_clear_draw( void )
{
    int x, y;
    int x1 = DEF_CLR_BASE_X1;
    int y1 = DEF_CLR_BASE_Y1;
    int color = 0;
    char *msg = "clear";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );
}


void  lcd_draw_erase_draw( void )
{
    int x, y;
    int x1 = DEF_ERS_BASE_X1;
    int y1 = DEF_ERS_BASE_Y1;
    int color = 5;
    char *msg = "erase";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );

    if( matled_erase_mode == 0 ) {
        color = 0;
    }
    for( x=0 ; x<10 ; x++ ) {
        for( y=0 ; y<10 ; y++ ) {
            lcd_pset( x1+x+7, y1+y+7, color );
        }
    }
}


void  lcd_draw_slide_draw( void )
{
    int x, y;
    int x1 = DEF_SLD_BASE_X1;
    int y1 = DEF_SLD_BASE_Y1;
    int color = 1;
    char *msg = "slide";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );

    if( matled_slide_mode == 0 ) {
        color = 0;
    }
    for( x=0 ; x<10 ; x++ ) {
        for( y=0 ; y<10 ; y++ ) {
            lcd_pset( x1+x+7, y1+y+7, color );
        }
    }
}


void  lcd_draw_copy_draw( void )
{
    int x, y;
    int x1 = DEF_CPY_BASE_X1;
    int y1 = DEF_CPY_BASE_Y1;
    int color = 3;
    char *msg = "copy";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );
}


void  lcd_draw_top_draw( void )
{
    int x, y;
    int x1 = DEF_TOP_BASE_X1;
    int y1 = DEF_TOP_BASE_Y1;
    int color = 4;
    char *msg = "top";

    for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
        lcd_pset( x1-1+x, y1-1, color );
        lcd_pset( x1-1+x, y1+DEF_COL_YD, color );
    }
    for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
        lcd_pset( x1-1, y1+y, color );
        lcd_pset( x1+DEF_COL_XD, y1+y, color );
    }

    set_string( x1+30, y1+5, color, msg, strlen(msg) );
}


void  lcd_draw_use_color( void )
{
    int x, y;

    for( x=0 ; x<DEF_COL_XD ; x++ ) {
        for( y=0 ; y<DEF_COL_YD ; y++ ) {
            lcd_pset( DEF_COL_USE_X1+x, DEF_COL_USE_Y1+y, lcd_dot_color );
        }
    }
}


void  lcd_draw_color_area( void )
{
    int x, y, c;

    for( c=0 ; c<8 ; c++ ) {
        for( x=0 ; x<DEF_COL_XD+2 ; x++ ) {
            lcd_pset( lcd_draw_col_pos[c].x-1+x, lcd_draw_col_pos[c].y-1, 6 );
            lcd_pset( lcd_draw_col_pos[c].x-1+x, lcd_draw_col_pos[c].y+DEF_COL_YD, 6 );
        }
        for( y=0 ; y<DEF_COL_YD+1 ; y++ ) {
            lcd_pset( lcd_draw_col_pos[c].x-1, lcd_draw_col_pos[c].y+y, 6 );
            lcd_pset( lcd_draw_col_pos[c].x+DEF_COL_XD, lcd_draw_col_pos[c].y+y, 6 );
        }
    }

    lcd_draw_clear_draw();
    lcd_draw_erase_draw();
    lcd_draw_slide_draw();
    lcd_draw_copy_draw();
    lcd_draw_top_draw();
}


void  lcd_draw_color( void )
{
    int x, y, c;

    for( c=0 ; c<8 ; c++ ) {
        for( x=0 ; x<DEF_COL_XD ; x++ ) {
            for( y=0 ; y<DEF_COL_YD ; y++ ) {
                lcd_pset( lcd_draw_col_pos[c].x+x, lcd_draw_col_pos[c].y+y, c );
            }
        }
    }

}


void  lcd_clear( void )
{
    unsigned short int  *dst = (void*)0xc0000000;
    int i, j;

    for( i=0 ; i<480 ; i++ ) {
        for( j=0 ; j<272 ; j++ ) {
            *dst++ = 0x0000;
        }
    }
}


void  lcd_check_key( int x, int y )
{
    int  c;
    int  nx, ny ;

    for( c=0 ; c<8 ; c++ ) {
        nx = lcd_draw_col_pos[c].x;
        ny = lcd_draw_col_pos[c].y;
        if( nx <= x && x <= (nx+DEF_COL_XD)
         && ny <= y && y <= (ny+DEF_COL_YD) ) {
            lcd_dot_color = c;
            lcd_draw_use_color();

            matled_erase_mode = 0;
            lcd_draw_erase_draw();
            break;
        }
    }

    nx = DEF_CLR_BASE_X1;
    ny = DEF_CLR_BASE_Y1;
    if( nx <= x && x <= (nx+DEF_COL_XD)
     && ny <= y && y <= (ny+DEF_COL_YD) ) {
        lcd_clear_drawall();
        lcd_draw_framearea();
    }

    nx = DEF_SLD_BASE_X1;
    ny = DEF_SLD_BASE_Y1;
    if( nx <= x && x <= (nx+DEF_COL_XD)
     && ny <= y && y <= (ny+DEF_COL_YD) ) {
        matled_set_slide();
    }

    nx = DEF_CPY_BASE_X1;
    ny = DEF_CPY_BASE_Y1;
    if( nx <= x && x <= (nx+DEF_COL_XD)
     && ny <= y && y <= (ny+DEF_COL_YD) ) {
        matled_copy_lcddraw();
        matled_copy_lcdframe();
    }

    nx = DEF_TOP_BASE_X1;
    ny = DEF_TOP_BASE_Y1;
    if( nx <= x && x <= (nx+DEF_COL_XD)
     && ny <= y && y <= (ny+DEF_COL_YD) ) {
        matled_disp_posi = 0;
    }

    nx = DEF_ERS_BASE_X1;
    ny = DEF_ERS_BASE_Y1;
    if( nx <= x && x <= (nx+DEF_COL_XD)
     && ny <= y && y <= (ny+DEF_COL_YD) ) {
        matled_set_erase();
        lcd_dot_color = 7;
        lcd_draw_use_color();
    }
}


